﻿using ProjectTank.Drawnings;
using ProjectTank.MovementStrategy;

namespace ProjectTank;
public partial class FormTank : Form
{
    /// <summary>
	/// Поле-объект для прорисовки объекта
	/// </summary>
    private DrawningTank2? _drawningTank2;

    /// <summary>
	/// Стратегия перемещения
	/// </summary>
	private AbstractStrategy? _strategy;

    /// <summary>
	/// Получение объекта 
	/// </summary>
	public DrawningTank2 SetTank
    {
        set
        {
            _drawningTank2 = value;
            _drawningTank2.SetPictureSize(pictureBoxTank.Width, pictureBoxTank.Height);
            comboBoxStrategy.Enabled = true;
            _strategy = null;
            Draw();
        }
    }

    /// <summary>
	/// Конструктор формы
	/// </summary>
    public FormTank()
    {
        InitializeComponent();
        _strategy = null;
    }

    /// <summary>
    /// Метод прорисовки машины
    /// </summary>
    private void Draw()
    {
        if (_drawningTank2 == null)
        {
            return;
        }
        Bitmap bmp = new(pictureBoxTank.Width, pictureBoxTank.Height);
        Graphics gr = Graphics.FromImage(bmp);
        _drawningTank2.DrawTransport(gr);
        pictureBoxTank.Image = bmp;
    }


    /// <summary>
    /// Перемещение объекта по форме (нажатие кнопок навигации)
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonMove_Click(object sender, EventArgs e)
    {
        if (_drawningTank2 == null)
        {
            return;
        }

        string name = ((Button)sender)?.Name ?? string.Empty;
        bool result = false;
        switch (name)
        {
            case "buttonUp":
                result = _drawningTank2.MoveTransport(DirectionType.Up);
                break;
            case "buttonDown":
                result = _drawningTank2.MoveTransport(DirectionType.Down);
                break;
            case "buttonLeft":
                result = _drawningTank2.MoveTransport(DirectionType.Left);
                break;
            case "buttonRight":
                result = _drawningTank2.MoveTransport(DirectionType.Right);
                break;
        }

        if (result)
        {
            Draw();
        }
    }

    /// <summary>
    /// Обработка нажатия кнопки "Шаг"
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonStrategyStep_Click(object sender, EventArgs e)
    {
        if (_drawningTank2 == null)
        {
            return;
        }

        if (comboBoxStrategy.Enabled)
        {
            _strategy = comboBoxStrategy.SelectedIndex switch
            {
                0 => new MoveToCenter(),
                1 => new MoveToBorder(),
                _ => null,
            };
            if (_strategy == null)
            {
                return;
            }
            _strategy.SetData(new MoveableTank2(_drawningTank2), pictureBoxTank.Width, pictureBoxTank.Height);
        }

        if (_strategy == null)
        {
            return;
        }

        comboBoxStrategy.Enabled = false;
        _strategy.MakeStep();
        Draw();

        if (_strategy.GetStatus() == StrategyStatus.Finish)
        {
            comboBoxStrategy.Enabled = true;
            _strategy = null;
        }
    }

    private void button1_Click(object sender, EventArgs e)
    {

    }

    private void pictureBoxTank_Click(object sender, EventArgs e)
    {

    }
}


