﻿using ProjectTank.Drawnings;
namespace ProjectTank.MovementStrategy;

/// <summary>
/// Класс-реализация IMoveableObject с использованием DrawningCar
/// </summary>
public class MoveableTank2 : IMoveableObject
{
    /// <summary>
    /// Поле-объект класса DrawningCar или его наследника
    /// </summary>
    private readonly DrawningTank2? _tank = null;

    /// <summary>
    /// Конструктор
    /// </summary>
    /// <param name="tank">Объект класса DrawningCar</param>
    public MoveableTank2(DrawningTank2 tank)
    {
        _tank = tank;
    }

    public ObjectParameters? GetObjectPosition
    {
        get
        {
            if (_tank == null || _tank.EntityTank2 == null || !_tank.GetPosX.HasValue || !_tank.GetPosY.HasValue)
            {
                return null;
            }
            return new ObjectParameters(_tank.GetPosX.Value, _tank.GetPosY.Value, _tank.GetWidth, _tank.GetHeight);
        }
    }

    public int GetStep => (int)(_tank?.EntityTank2?.Step ?? 0);

    public bool TryMoveObject(MovementDirection direction)
    {
        if (_tank == null || _tank.EntityTank2 == null)
        {
            return false;
        }

        return _tank.MoveTransport(GetDirectionType(direction));
    }

    /// <summary>
    /// Конвертация из MovementDirection в DirectionType
    /// </summary>
    /// <param name="direction">MovementDirection</param>
    /// <returns>DirectionType</returns>
    private static DirectionType GetDirectionType(MovementDirection direction)
    {
        return direction switch
        {
            MovementDirection.Left => DirectionType.Left,
            MovementDirection.Right => DirectionType.Right,
            MovementDirection.Up => DirectionType.Up,
            MovementDirection.Down => DirectionType.Down,
            _ => DirectionType.Unknow,
        };
    }
}